package Padre::Plugin::Moose::Role::CanProvideHelp;

use Moose::Role;
use namespace::clean;

our $VERSION = '0.17';

requires 'provide_help';

1;

__END__

=pod

=head1 NAME

Padre::Plugin::Moose::Role::CanProvideHelp - Something that can provide help about itself

=cut
